@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2000-2011 Greg Sabino Mullane and others: see the Changes file
+    Copyright (c) 2000-2013 Greg Sabino Mullane and others: see the Changes file
 	Portions Copyright (c) 1997-2000 Edmund Mergl
 	Portions Copyright (c) 1994-1997 Tim Bunce
 	
@@ -23,6 +23,7 @@ struct imp_dbh_st {
 	int     copystate;         /* 0=none PGRES_COPY_IN PGRES_COPY_OUT */
 	int     pg_errorlevel;     /* PQsetErrorVerbosity. Set by user, defaults to 1 */
 	int     server_prepare;    /* do we want to use PQexecPrepared? 0=no 1=yes 2=smart. Can be changed by user */
+	int     switch_prepared;   /* how many executes until we switch to PQexecPrepared */
 	int     async_status;      /* 0=no async 1=async started -1=async has been cancelled */
 
     imp_sth_t *async_sth;      /* current async statement handle */
@@ -30,14 +31,17 @@ struct imp_dbh_st {
 	PGconn  *conn;             /* connection structure */
 	char    *sqlstate;         /* from the last result */
 
+
 	bool    pg_bool_tf;        /* do bools return 't'/'f'? Set by user, default is 0 */
-    bool    utf8_strings;      /* so we set the utf8 flag on data from the database? */
-	bool    pg_enable_utf8;    /* (DEPRECATED) should we attempt to make utf8 strings? Set by user, default is 0 */
 	bool    prepare_now;       /* force immediate prepares, even with placeholders. Set by user, default is 0 */
 	bool    done_begin;        /* have we done a begin? (e.g. are we in a transaction?) */
 	bool    dollaronly;        /* only consider $1, $2 ... as valid placeholders */
 	bool    expand_array;      /* transform arrays from the db into Perl arrays? Default is 1 */
 	bool    txn_read_only;     /* are we in read-only mode? Set with $dbh->{ReadOnly} */
+
+	int     pg_enable_utf8;    /* legacy utf8 flag: force utf8 flag on or off, regardless of client_encoding */
+	bool    pg_utf8_flag;      /* are we currently flipping the utf8 flag on? */
+    bool    client_encoding_utf8; /* is the client_encoding utf8 last we checked? */
 };
 
 
@@ -72,15 +76,17 @@ typedef struct ph_st ph_t;
 struct imp_sth_st {
 	dbih_stc_t com;          /* MUST be first element in structure */
 
-	int    server_prepare;   /* inherited from dbh. 3 states: 0=no 1=yes 2=smart */
-	int    placeholder_type; /* which style is being used 1=? 2=$1 3=:foo */
-	int    numsegs;          /* how many segments this statement has */
-	int    numphs;           /* how many placeholders this statement has */
-	int    numbound;         /* how many placeholders were explicitly bound by the client, not us */
-	int    cur_tuple;        /* current tuple being fetched */
-	int    rows;             /* number of affected rows */
-	int    async_flag;       /* async? 0=no 1=async 2=cancel 4=wait */
-	int    async_status;     /* 0=no async 1=async started -1=async has been cancelled */
+	int    server_prepare;    /* inherited from dbh. 3 states: 0=no 1=yes 2=smart */
+	int    switch_prepared;   /* inherited from dbh */
+    int    number_iterations; /* how many times has the statement been executed? Used by switch_prepared */
+	int    placeholder_type;  /* which style is being used 1=? 2=$1 3=:foo */
+	int    numsegs;           /* how many segments this statement has */
+	int    numphs;            /* how many placeholders this statement has */
+	int    numbound;          /* how many placeholders were explicitly bound by the client, not us */
+	int    cur_tuple;         /* current tuple being fetched */
+	int    rows;              /* number of affected rows */
+	int    async_flag;        /* async? 0=no 1=async 2=cancel 4=wait */
+	int    async_status;      /* 0=no async 1=async started -1=async has been cancelled */
 
 	STRLEN totalsize;        /* total string length of the statement (with no placeholders)*/
 
@@ -189,11 +195,11 @@ int pg_db_getfd (imp_dbh_t * imp_dbh);
 
 SV * pg_db_pg_notifies (SV *dbh, imp_dbh_t *imp_dbh);
 
-SV * pg_stringify_array(SV * input, const char * array_delim, int server_version, int extraquotes);
+SV * pg_stringify_array(SV * input, const char * array_delim, int server_version);
 
 int pg_quickexec (SV *dbh, const char *sql, const int asyncflag);
 
-int pg_db_putline (SV *dbh, const char *buffer);
+int pg_db_putline (SV *dbh, SV *svbuf);
 
 int pg_db_getline (SV *dbh, SV * svbuf, int length);
 
@@ -229,6 +235,8 @@ int pg_db_lo_lseek (SV *dbh, int fd, int offset, int whence);
 
 int pg_db_lo_tell (SV *dbh, int fd);
 
+int pg_db_lo_truncate (SV *dbh, int fd, size_t len);
+
 int pg_db_lo_unlink (SV *dbh, unsigned int lobjId);
 
 unsigned int pg_db_lo_import (SV *dbh, char *filename);
